package ${basePackage}.action.ajax.system;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import ${basePackage}.frame.auth.LocalData;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.utils.CookieUtil;
import ${basePackage}.frame.utils.MD5Util;
import ${basePackage}.frame.utils.RequestUtil;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.User;
import ${basePackage}.module.system.mgr.TokensManager;
import ${basePackage}.module.system.mgr.UserManager;
import ${basePackage}.module.system.req.AuthLoginRequest;
import ${basePackage}.module.system.req.TokensCreateRequest;
import ${basePackage}.module.system.req.UserFindRequest;
import ${basePackage}.module.system.rsp.AuthLoginResponse;
import ${basePackage}.module.system.rsp.TokensCreateResponse;
import ${basePackage}.module.system.rsp.UserFindResponse;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Calendar;
import java.util.Date;
import java.util.UUID;

public class AuthAjax {
    @Value("${r'${web.url.auth.admin}'}")
    private String admin;
    @Value("${r'${web.url.auth.pwd}'}")
    private String pwd;
    @Autowired
    private TokensManager tokensManager;
    @Autowired
    private UserManager userManager;

    public AuthLoginResponse login(AuthLoginRequest request, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        AuthLoginResponse response = new AuthLoginResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        if (request.getUsername().equals(admin)) {//超级管理员登录
            String generatePwd = MD5Util.generatePwd(request.getPassword());
            if (!generatePwd.equals(pwd)) {
                response.addError(ErrorType.BUSINESS_ERROR, "登录密码错误");
            } else {
                Date current = new Date();
                Calendar instance = Calendar.getInstance();
                instance.setTime(current);
                instance.add(Calendar.HOUR_OF_DAY, 1);//默认一个小时内有效

                TokensCreateRequest tokensCreateRequest = new TokensCreateRequest();
                tokensCreateRequest.setToken(UUID.randomUUID().toString());
                tokensCreateRequest.setUserId(0L);
                tokensCreateRequest.setUserName("超级管理员");
                tokensCreateRequest.setLoginTime(current);
                tokensCreateRequest.setValidTime(instance.getTime());
                tokensCreateRequest.setValid(true);
                tokensCreateRequest.setDeptId(null);
                tokensCreateRequest.setDeptCode(null);
                tokensCreateRequest.setDeptName(null);
                tokensCreateRequest.setTerminalType("WEB");
                tokensCreateRequest.setTerminalIp(RequestUtil.getIp(httpServletRequest));
                tokensCreateRequest.setTerminalInfo(RequestUtil.getUserAgent(httpServletRequest));
                TokensCreateResponse tokensCreateResponse = tokensManager.create(tokensCreateRequest, LocalData.getSysToken());

                if (tokensCreateResponse.hasError()) {
                    response.addErrors(tokensCreateResponse.getErrors());
                    return response;
                }
                Cookie cookie = CookieUtil.newCookie("token", tokensCreateRequest.getToken());
                cookie.setPath("/");
                httpServletResponse.addCookie(cookie);
            }
        } else {//管理员登录
            UserFindRequest userFindRequest = new UserFindRequest();
            userFindRequest.setUserName(request.getUsername());
            UserFindResponse userFindResponse = userManager.find(userFindRequest, LocalData.getSysToken());

            if (userFindResponse.hasError()) {
                response.addErrors(userFindResponse.getErrors());
                return response;
            } else if (userFindResponse.getResult().size() == 0) {
                response.addError(ErrorType.BUSINESS_ERROR, "用户名或密码错误!");
                return response;
            } else if (userFindResponse.getResult().size() > 1) {
                response.addError(ErrorType.BUSINESS_ERROR, "用户信息异常!");
                return response;
            }

            User user = userFindResponse.getResult().get(0);

            String generatePwd = MD5Util.generatePwd(request.getPassword());
            if (!generatePwd.equals(user.getUserPwd())) {
                response.addError(ErrorType.BUSINESS_ERROR, "用户名或密码错误!");
                return response;
            }

            Date current = new Date();
            Calendar instance = Calendar.getInstance();
            instance.setTime(current);
            instance.add(Calendar.HOUR_OF_DAY, 1);//默认一个小时内有效

            TokensCreateRequest tokensCreateRequest = new TokensCreateRequest();
            tokensCreateRequest.setToken(UUID.randomUUID().toString());
            tokensCreateRequest.setUserId(user.getId());
            tokensCreateRequest.setUserName(user.getUserName());
            tokensCreateRequest.setLoginTime(current);
            tokensCreateRequest.setValidTime(instance.getTime());
            tokensCreateRequest.setValid(true);
            tokensCreateRequest.setDeptId(user.getDeptId());
            tokensCreateRequest.setDeptCode(user.getDeptCode());
            tokensCreateRequest.setDeptName(user.getDeptName());
            tokensCreateRequest.setTerminalType("WEB");
            tokensCreateRequest.setTerminalIp(RequestUtil.getIp(httpServletRequest));
            tokensCreateRequest.setTerminalInfo(RequestUtil.getUserAgent(httpServletRequest));
            TokensCreateResponse tokensCreateResponse = tokensManager.create(tokensCreateRequest, LocalData.getSysToken());

            if (tokensCreateResponse.hasError()) {
                response.addErrors(tokensCreateResponse.getErrors());
                return response;
            }
            Cookie cookie = CookieUtil.newCookie("token", tokensCreateRequest.getToken());
            cookie.setPath("/");
            httpServletResponse.addCookie(cookie);
        }

        return response;
    }
}
