package ${basePackage}.action.ajax.system;

import com.fasterxml.jackson.core.TreeNode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.multipart.MultipartFile;
import ${basePackage}.frame.excel.WSheet;
import ${basePackage}.frame.utils.LogUtil;
import ${basePackage}.frame.auth.LocalData;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.base.BaseResponse;
import ${basePackage}.frame.utils.ResponseUtil;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.frame.excel.exception.ReadErrorException;
import ${basePackage}.frame.excel.exception.TemplateNotMatchException;
import ${basePackage}.module.system.ent.User;
import ${basePackage}.module.system.mgr.UserManager;
import ${basePackage}.module.system.req.UserCreateRequest;
import ${basePackage}.module.system.req.UserDeleteRequest;
import ${basePackage}.module.system.req.UserFindRequest;
import ${basePackage}.module.system.req.UserGetRequest;
import ${basePackage}.module.system.req.UserUpdateRequest;
import ${basePackage}.module.system.rsp.UserCreateResponse;
import ${basePackage}.module.system.rsp.UserDeleteResponse;
import ${basePackage}.module.system.rsp.UserFindResponse;
import ${basePackage}.module.system.rsp.UserGetResponse;
import ${basePackage}.module.system.rsp.UserUpdateResponse;

import java.io.IOException;
import java.util.List;

public class UserAjax{

    @Autowired
    private UserManager userManager;

    public UserCreateResponse create(UserCreateRequest request) {
        return userManager.create(request, LocalData.getToken());
    }

    public UserDeleteResponse delete(UserDeleteRequest request) {
        return userManager.delete(request, LocalData.getToken());
    }

    public UserUpdateResponse update(UserUpdateRequest request) {
        return userManager.update(request, LocalData.getToken());
    }

    public UserFindResponse find(UserFindRequest request) {
        return userManager.find(request, LocalData.getToken());
    }

    public UserGetResponse get(UserGetRequest request) {
        return userManager.get(request, LocalData.getToken());
    }

    public Object template(){
        return ResponseUtil.apply(new WSheet<>(User.class));
    }

    public Object exports(TreeNode jsonParam) {
        UserFindRequest request = MapperUtil.toJava(jsonParam, UserFindRequest.class);
        UserFindResponse response = userManager.find(request, LocalData.getToken());
        if (response.hasError()) {
            return response;
        } else if (response.getTotalCount() == 0) {
            response.addError(ErrorType.BUSINESS_ERROR, "导出数据为空");
            return response;
        }
        return ResponseUtil.apply(new WSheet<>(response.getResult()));
    }

    public Object imports(MultipartFile file) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            WSheet<User> sheet = new WSheet<>(file.getBytes(), User.class, new WSheet.Validator<User>() {
                @Override
                public List<String> validate(User o) {
                    UserCreateRequest request = MapperUtil.map(o, UserCreateRequest.class);
                    return ValidationUtil.validate(request);
                }
            });

            if (sheet.hasError()) {
                return ResponseUtil.apply(sheet.getBytes(), sheet.getName() + "-检查.xlsx");
            } else {
                return baseResponse;
            }
        } catch (IOException e) {
            e.printStackTrace();
            LogUtil.dumpException(e);
            baseResponse.addError(ErrorType.BUSINESS_ERROR, "上传文件出错");
        } catch (TemplateNotMatchException | ReadErrorException e) {
            baseResponse.addError(ErrorType.BUSINESS_ERROR, e.getMessage());
        }
        return baseResponse;
    }
}
